#ifndef FUNCTION_H_
#define FUNCTION_H_

// Contains IFunction class and some standard
// implementations of IFunction.

#include <list>
#include "Defs.h"

class Cell;

// The combination of Cell and Function
// implements the State pattern [GOF].
// The Function represents the primary
// state of the Cell.

// Interface class representing an 
// arbitrary function
class IFunction
{
public:
    virtual ~IFunction();
    virtual Value evaluate() const =0;
    virtual std::list<Cell*> getSources() = 0;
};

// Class representing a binary operator
// function such as +, *, etc.
// The exact function is specified at
// construction time via a function
// pointer.
class BinOp : public IFunction
{
public:
    typedef Value (*FuncPtr)(Value, Value);

    BinOp(FuncPtr,Cell *arg1, Cell *arg2);
    virtual ~BinOp();

    // Implementation of IFunction:
    virtual Value evaluate() const;
    virtual std::list<Cell*> getSources();
private:
    FuncPtr func;
    Cell *lhs;
    Cell *rhs;
};

// (BinOp also represents a use of the
// Adapter pattern, in that it adapts
// the free functions below to conform
// to the IFunction interface.)

// A standard library of common binary
// functions for use with BinOp:
Value add(Value,Value);
Value subtract(Value,Value);
Value multiply(Value,Value);
Value divide(Value,Value);
Value modfunc(Value,Value);

#endif
